note
	description: "[
				A VIEW to be used to open or edit INTERFACE_NAMES and then
				save those changes or apply the changes to the current
				INTERFACE_TABLE	in MAIN_WINDOWs from the "Standard Interface"
				cluster.
				]"
	author: "Jimmy J. Johnson"
	date: "23 Dec 03"

class
	INTERFACE_TABLE_VIEW

inherit

	EV_MULTI_COLUMN_LIST
		rename
			object_id as ise_object_id
		redefine
			initialize,
			destroy,
			is_destroyed
		end

	VIEW
		rename
			target as table,
			set_target as set_table
		undefine
			copy,
			is_equal
		redefine
			initialize,
			table,
			set_table,
			draw,
			destroy,
			is_destroyed
		end


create
	default_create

feature {NONE} -- Initialization

	initialize
			-- Create an editor view.
		do
			Precursor {EV_MULTI_COLUMN_LIST}
			Precursor {VIEW}
			set_column_title ("Widget Type/Name", 1)
			set_column_title ("Menu Item", 2)
			set_column_title ("Tooltip", 3)
			set_column_title ("Help Text", 4)
			set_column_title ("Pixmap file", 5)
			set_actions
		end

	set_actions
			-- Add actions to the widgets
		do
			column_title_click_actions.extend (agent on_column_title_clicked)
		end

feature -- Access

	table: INTERFACE_TABLE
			-- The object contained in this view.
			-- Redefined to change type

feature -- Element change

	set_table (a_table: like table)
			-- Change `table'
		do
			Precursor {VIEW} (a_table)
			draw
		end

feature -- Basic operations

	draw
			-- Put the interface table items into the view
		local
			ii: INTERFACE_ITEM
			row: EV_MULTI_COLUMN_LIST_ROW
		do
				-- Clean out the viewables from the rows
			wipe_out
			from table.names.start
			until table.names.off
			loop
				ii := table.names.item_for_iteration
				create row
				row.extend (table.names.key_for_iteration)
				row.set_pebble (ii)
				row.extend (ii.short_name)
				row.extend (ii.tooltip)
				row.extend (ii.help_text)
				row.extend (ii.pixmap_file)
				row.drop_actions.extend (agent on_drop_pixmap_file)
				extend (row)
				table.names.forth
			end
		end

	destroy
			-- Destroy underlying native toolkit object.
			-- Render `Current' unusable.
		do
			Precursor {VIEW}
			Precursor {EV_MULTI_COLUMN_LIST}
		end

feature -- Status report

	is_destroyed: BOOLEAN
			-- Is `Current' no longer usable?
		do
			Result := Precursor {VIEW} and Precursor {EV_MULTI_COLUMN_LIST}
		end

feature {NONE} -- Implementation (actions)

	on_drop_pixmap_file (a_string: STRING)
			-- React to a drop of a string on a row.
			-- Note:  this action is added to each row in feature `draw'.
		local
			ii: INTERFACE_ITEM
			r: EV_MULTI_COLUMN_LIST_ROW
		do
			r := selected_item
			if r /= Void then
				ii ?= r.pebble
				check
					item_exists: ii /= Void
						-- because all rows have INTERFACE_ITEM as the pebble
				end
				ii.set_pixmap_file (a_string)
				r.put_i_th (a_string, 5)
--				draw_views (ii)
			end
		end

	on_column_title_clicked (a_column: INTEGER)
			-- React to a click on a column title by sorting the items
			-- based on the string in this column.
		require
			column_number_large_enough: a_column >= 1
			column_number_small_enough: a_column <= column_count
		local
			r_list: LINKED_LIST [EV_MULTI_COLUMN_LIST_ROW]
			r, smallest_r: EV_MULTI_COLUMN_LIST_ROW
		do
			create r_list.make
				-- Put the rows into a temp list in ascending order
			from start
			until is_empty
			loop
				smallest_r := item
				from start
				until exhausted
				loop
					r := item
					if r.i_th (a_column) < smallest_r.i_th (a_column) then
						smallest_r := r
					end
					forth
				end
				prune (smallest_r)
				r_list.extend (smallest_r)
				start
			end
			from r_list.start
			until r_list.is_empty
			loop
				r := r_list.item
				r_list.remove
				extend (r)
			end
		end

--	on_filename_dropped (a_name: STRING) is
--			-- React to a drop of a STRING, assuming that `a_name' is the
--			-- name of a icon or pixmap file.
--		require
--			name_exists: a_name /= Void
--		do
--			
--		end

	on_row_selected
			-- React to a row selection by finding the corresponding `interface_item'
			-- from the table, thus allowing the correct item to be editted if needed.
		local
			r: EV_MULTI_COLUMN_LIST_ROW
			k: STRING
		do
			interface_item_imp := Void
				-- Get the currently selected row from Current
			r := selected_item
				-- The first column of Current (therefore the first item
				-- in the row) is used as a key in the table.
			k := r.i_th (1)
				-- Find the corresponding interface item.
			interface_item_imp := table.interface_item (k)
		ensure
				-- if row was selected there must be a corresponding item in the table.
			item_found: interface_item_imp /= Void
		end

feature {NONE} -- Implementation

	column_at_x (x: INTEGER): INTEGER
			-- The column containing horizontal coordinate `x'.
		local
			x_off: INTEGER
			i: INTEGER
		do
			from i := 1
			until i > column_count or else x_off + column_width (i) > x
			loop
				x_off := x_off + column_width (i)
				i := i + 1
			end
			Result := i
		end

	row_at_y (y: INTEGER): INTEGER
			-- The row containing the vertical coordinate `y'.
		local
			y_off: INTEGER
			i: INTEGER
		do
			from i := 1
			until i >= count or else y_off + row_height > y
			loop
				y_off := y_off + row_height
				i := i + 1
			end
			Result := i
		end

	column_position (a_column_number: INTEGER): INTEGER
			-- The x position of column `a_column_number' in screen coordinates.
			-- Used to position the edit dialog box.

		require
			column_number_large_enough: a_column_number >= 1
			column_number_small_enough: a_column_number <= column_count
		local
			i: INTEGER
		do
			Result := screen_x
			from i := 1
			until i >= a_column_number - 1
			loop
				Result := Result + column_width (i)
				i := i + 1
			end
		end

	row_position (a_row_number: INTEGER): INTEGER
			-- The y position of row `a_row_number' in screen coordinates.
			-- Used to position the edit dialog box.
		require
			row_number_large_enough: a_row_number >= 1
			row_number_small_enough: a_row_number <= count
		do
			Result := screen_y
			Result := Result + a_row_number * row_height
		end


	interface_item_imp: INTERFACE_ITEM
			-- Used internally to hold on to the item currently being editted, so
			-- it can be changed when an "end edit" action occurs.

end -- class INTERFACE_TABLE_VIEW
