note
	description: "[
		A binary-like tree for holding "siblings" or "splits" for use by
		SPLIT_MANAGER.  The tree manages the placement of exactly two children
		into a node of the tree.
		]"
	author: "Jimmy J. Johnson"
	date: "17 Nov 07"

deferred class
	S_TREE [G]

inherit

	ANY
		redefine
			default_create
		end

feature {NONE} -- Initialization

	default_create
			-- Create an empty tree.
		do
		end

	make (a_item: G)
			-- Make a new tree whose `root' node contains `a_item'.
		require
			item_exists: a_item /= Void
		do
			create root.make (a_item)
			last_node := root
		end

feature -- Access

	root: S_TREE_NODE [like item]
		-- The root of the tree.

	root_item: like item
			-- The item in the root node.
		do
			Result := root.item
		ensure
			result_exists: Result /= Void
		end

	last_item: like item
			-- The `item' in the parent node created when two new nodes are
			-- added with a call to `extend'.
		do
			Result := last_node.item
		end

	descendents: LINKED_SET [like root]
			-- All the nodes descended from `root' "post-order" (first, second, Current)
		do
			if root = Void then
				create Result.make
			else
				Result := root.descendents
			end
		end

	internal_nodes: LINKED_SET [like root]
			-- All the descendent nodes of Current which are not leaf (`is_empty') nodes
		do
			if root = Void then
				create Result.make
			else
				Result := root.internal_nodes
			end
		end

	leaf_nodes: LINKED_SET [like root]
			-- All the descendent nodes which are leaf (`is_empty') nodes.
		do
			if root = Void then
				create Result.make
			else
				Result := root.leaf_nodes
			end
		end

feature -- Element change

	set_root_item (a_item: like item)
			-- Change the `item' of `root'.
		require
			item_exists: a_item /= Void
		do
			root.set_item (a_item)
		end

	set_root (a_node: like root)
			-- Set `root' to `a_root'.
		require
			node_exists: a_node /= Void
		do
			root := a_node
		end

feature -- Measurement

	count: INTEGER
			-- The number of nodes in the tree.
		do
			Result := root.descendents.count
		end

	leaf_count: INTEGER
			-- The number of leaf nodes under `root'.
		do
			Result := root.leaf_nodes.count
		end

feature -- Basic operations

--	test_print is
--			--
--		do
--			root.test_print (0)
--			io.new_line
--			io.new_line
--		end

	extend (a_item: like item)
			-- Add `a_item' as a sibling to the previously added item.
		require
			item_exists: a_item /= Void
			not_has_item: not has (a_item)
		local
			p, n, internal_n: like root
		do
			create n.make (a_item)
			if last_node = Void then
					-- Add as root
				root := n
			elseif last_node.parent = Void then
				create p.make (new_item)
				p.set_children (last_node, n)
				root := p
			else
				p := last_node.parent
				create internal_n.make (new_item)
				internal_n.set_children (p, n)
			end
			last_node := n
		ensure
			has_item: has (a_item)
		end

	extend_siblings (a_first, a_second: like item)
			-- Searching from `root', find the nodes containing `a_first', and/or
			-- `a_second' or, if not finding them, create new nodes containing
			-- `a_first', and `a_second' and make them children of a new node.
			--  Set `last_node' to the new node.
		require
			first_exists: a_first /= Void
			second_exists: a_second /= Void
			not_same: a_first /= a_second
			has_both_only_if_siblings: (has (a_first) and has (a_second)) implies is_siblings (a_first, a_second)
		local
			n1, n2: like root
			p, other_n, new_internal_n: like root
			n: like root
		do
			if has (a_first) and has (a_second)then
				n1 := corresponding_node (a_first)
				n2 := corresponding_node (a_second)
				check
					is_siblings: is_siblings (a_first, a_second)
						-- Because of precondition "has_both_only_if_siblings"
				end
					-- The nodes are siblings already in tree so simply change the
					-- `item' contained in the `parent' node of `n1' and `n2' to
					-- ensure it matches the type set by `new_item'.
				n1.parent.set_item (new_item)
				last_node := n1.parent
			elseif has (a_first) or has (a_second) then
				create new_internal_n.make (new_item)
				if has (a_first) then
					check
						not_has_second: not has (a_second)
							-- because of previouse if statement.
					end
					n1 := corresponding_node (a_first)
					create n2.make (a_second)
				else
					check
						has_second: has (a_second)
							-- because of previouse if statement.
					end
					n1 := corresponding_node (a_second)
					create n1.make (a_first)
				end
				if n1.is_orphan then
					check
						is_root: n1 = root
							-- Only node without a parent should be the root.
					end
					root := new_internal_n
					last_node := new_internal_n
				else
					p := n1.parent
					other_n := p.other_node (n1)
					if n1.is_first (p) then
						p.set_children (new_internal_n, other_n)
					else
						p.set_children (other_n, new_internal_n)
					end
				end
				new_internal_n.set_children (n1, n2)
			else		-- neither `a_first' or `a_second' is already in tree.
				create new_internal_n.make (new_item)
				create n1.make (a_first)
				create n2.make (a_second)
				new_internal_n.set_children (n1, n2)
				last_node := new_internal_n
				if root = Void or else root.is_empty then
					root := new_internal_n
				else
					n := root
					create root.make (new_item)
					root.set_children (n, new_internal_n)
				end
			end
		ensure
			root_exists: root /= Void
--			new_root: root /= old root
			root_is_full: root.is_full
--			root_has_first_item: root.first.item = a_first
--			root_has_second_item: root.second.item = a_second
		end

feature -- Status report

	is_empty: BOOLEAN
			-- Is the tree empty?
		do
			Result := root = Void
		end

feature -- Status setting

feature -- Querry

	has (a_item: like item): BOOLEAN
			-- Does Current have a node containing `a_item'?
		require
			item_exists: a_item /= Void
		local
			s: LINKED_SET [S_TREE_NODE [G]]
		do
			if root /= Void then
				s := root.descendents
				from s.start
				until Result or else s.exhausted
				loop
					Result := s.item.item = a_item
					s.forth
				end
			end
		end

	has_node (a_node: like root): BOOLEAN
			-- Does the tree contain `a_node'?
		require
			node_exists: a_node /= Void
		local
			s: like descendents
		do
			if root /= Void then
				s := root.descendents
				from s.start
				until Result or else s.exhausted
				loop
					Result := s.item = a_node
					s.forth
				end
			end
		end

	corresponding_node (a_item: like item): like root
			-- The node containing `a_item'.
		require
			node_exists: a_item /= Void
			has_item: has (a_item)
		local
			s: LINKED_SET [S_TREE_NODE [G]]
			n: S_TREE_NODE [G]
		do
			s := root.descendents
			from s.start
			until Result /= Void or else s.exhausted
			loop
				n := s.item
				if n.item = a_item then
					Result := n
				end
				s.forth
			end
		end

	is_siblings (a_first, a_second: like item): BOOLEAN
			-- Are `a_first' and `a_second' items of nodes which are children of a common parent?
		require
			first_exists: a_first /= Void
			second_exists: a_second /= Void
		local
			n1, n2: like root
		do
			if has (a_first) and has (a_second) then
				n1 := corresponding_node (a_first)
				n2 := corresponding_node (a_second)
				Result := n1.parent = n2.parent
			end
		end

feature {NONE} -- Implementation

	item: G
			-- Anchor for the generic.

	last_node: like root
			-- The last node created while building the tree.  To give
			-- access to the last item.

	new_item: like item
			-- Create a new `item'.  Called by `extend'.
		deferred
		ensure
			result_exists: Result /= Void
		end

	has_invalid_node: BOOLEAN
			-- Does Current contain a node whose `item' is Void
		local
			s: like descendents
			n: like root
		do
			s := root.descendents
			from s.start
			until Result or else s.exhausted
			loop
				n := s.item
				Result := n.item = Void
				s.forth
			end
		end

invariant

	all_nodes_have_an_item: root /= Void implies not has_invalid_node

end
