note
	description: "[
		Binary nodes of an S_TREE (used by SPLIT_MANAGER).
		]"
	author: "Jimmy J. Johnson"
	date: "17 Nov 07"

class
	S_TREE_NODE [G]

inherit

	ANY
		redefine
--			default_create
		end

create
	make

feature {NONE} -- Initialization

	make (a_item: like item)
			-- Create a node holding `a_item' in `item'.
		require
			item_exists: a_item /= Void
		do
			item := a_item
		ensure
			item_set: item = a_item
		end

feature -- Access

	item: G
			-- The item in this node.

	first: like Current
			-- First child.

	second: like Current
			-- Second child.

	parent: like Current
			-- Parent of which Current is a child.

	other_node (a_node: like Current): like Current
			-- The child of current which is not `a_node'.
		require
			node_exists: a_node /= Void
			has_node: has (a_node)
		do
			if a_node = first then
				Result := second
			else
				Result := first
			end
		ensure
			result_exists: Result /= Void
			result_is_child: Result.parent = Current
		end

	descendents: LINKED_SET [like Current]
			-- All the nodes descended from Current "post-order" (first, second, Current)
		do
			create Result.make
			if is_full then
				Result.merge (first.descendents)
				Result.merge (second.descendents)
			end
			Result.extend (Current)
		end

	internal_nodes: LINKED_SET [like Current]
			-- All the descendent nodes of Current which are not leaf (`is_empty') nodes
		do
			create Result.make
			if is_full then
				Result.merge (first.internal_nodes)
				Result.merge (second.internal_nodes)
				Result.extend (Current)
			end
		end

	leaf_nodes: LINKED_SET [like Current]
			-- All the descendent nodes which are leaf (`is_empty') nodes.
		do
			create Result.make
			if is_empty then
				Result.extend (Current)
			else
				Result.merge (first.leaf_nodes)
				Result.merge (second.leaf_nodes)
			end
		end

feature -- Element change

	set_item (a_item: like item)
			-- Make Current hold `a_item'.
		require
			item_exists: a_item /= Void
		do
			item := a_item
		ensure
			item_set: item = a_item
		end

	remove_item
			-- Make sure `item' is Void
		local
			l_default_value: like item
		do
--			item := Void
			item := l_default_value
		ensure
			item_removed: item = Void
		end

	set_children (a_first, a_second: like Current)
			-- Change `first'.
		require
			first_exists: a_first /= Void
			second_exists: a_second /= Void
		do
			first := a_first
			second := a_second
			a_first.set_parent (Current)
			a_second.set_parent (Current)
		ensure
			first_set: first = a_first
			second_set: second = a_second
			first_parent_set: a_first.parent = Current
			second_parent_set: a_second.parent = Current
			is_full: is_full
		end

	set_first (a_first: like Current)
			-- Change `first'
		require
			first_exists: a_first /= Void
			is_full: is_full
		do
			first := a_first
			first.set_parent (Current)
		ensure
			first_set: first = a_first
			first_parent_set: first.parent = Current
		end

	set_second (a_second: like Current)
			-- Change `second'
		require
			second_exists: a_second /= Void
			is_full: is_full
		do
			second := a_second
			second.set_parent (Current)
		ensure
			second_set: second = a_second
			second_parent_set: second.parent = Current
		end

	remove_children
			-- Make sure current has no children.
		require
			is_full: is_full
		local
			f, s: S_TREE_NODE [G]
		do
			f := first
			s := second
			first := Void
			second := Void
			f.remove_parent
			s.remove_parent
		ensure
			is_empty: is_empty
			first_is_orphaned: old first.is_orphan
			second_is_orphaned: old second.is_orphan
		end

feature -- Basic operations

	test_print (indent: INTEGER)
			-- Output for testing
		local
			i: INTEGER
		do
			from i := 1
			until i > indent
			loop
				io.put_string ("%T")
				i := i + 1
			end
			io.put_string ("item = " + item.out)
			io.new_line
			if first /= Void then
				first.test_print (indent + 1)
			end
			if second /= Void then
				second.test_print (indent + 1)
			end
		end


feature -- Status report

	is_full: BOOLEAN
			-- Does Current have two children?
		do
			Result := first /= Void and second /= Void
		ensure
			definition: Result implies first /= Void and second /= Void
		end

	is_empty: BOOLEAN
			-- Does Current have zero children?
		do
			Result := first = Void and second = Void
		ensure
			definition: Result implies first = Void and second = Void
		end

	is_orphan: BOOLEAN
			-- Does Current has no parent?
		do
			Result := parent = Void
		ensure
			definition: Result = (parent = Void)
		end

feature -- Querry

	is_child_of (a_node: S_TREE_NODE [G]): BOOLEAN
			-- Is Current a child of `a_node'?
		require
			node_exists: a_node /= Void
		do
			Result := parent = a_node
		ensure
			definition: Result implies parent = a_node
		end

	has (a_node: S_TREE_NODE [G]): BOOLEAN
			-- Is `a_node' a child of Current?
		require
			node_exists: a_node /= Void
		do
			Result := first = a_node or else second = a_node
		ensure
			definition: Result implies first = a_node or else second = a_node
		end

	is_first (a_node: S_TREE_NODE [G]): BOOLEAN
			-- Is Current the first child of `a_node'?
		require
			node_exists: a_node /= Void
		do
			Result := a_node.first = Current
		ensure
			definition: Result implies a_node.first = Current
		end

	is_second (a_node: S_TREE_NODE [G]): BOOLEAN
			-- Is Current the second child of `a_node'?
		require
			node_exists: a_node /= Void
		do
			Result := a_node.second = Current
		ensure
			definition: Result implies a_node.second = Current
		end

	is_sibling (a_node: S_TREE_NODE [G]): BOOLEAN
			-- Is `a_node' a sibling of Current?  (I.e. is Current and `a_node'
			-- both children of the same node?
		require
			node_exists: a_node /= Void
		do
			Result := a_node.parent = parent
		ensure
			definition: Result implies a_node.parent = parent
		end

feature {S_TREE_NODE} -- Implementation

	set_parent (a_node: S_TREE_NODE [G])
			-- Make `a_node' the parent of Current.
		require
			node_exists: a_node /= Void
			has_child: a_node.has (Current)
		do
			parent := a_node
		ensure
			parent_set: parent = a_node
		end

	remove_parent
			-- Make sure current has no parent.
		require
			not_is_orphan: not is_orphan
			not_has_child: parent.is_empty
		do
			parent := Void
		ensure
			is_orphan: is_orphan
		end

invariant

	zero_or_two_children: is_full or is_empty

end
